﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Interconnection
{
    public class Codebooks
    {

        public IDictionary<int, Association> dAssociations { get; set; }
        public IDictionary<int, Category> dCategories { get; set; }
        public IDictionary<int, TimerType> dTypeTimers { get; set; }
        public IList<string> lLegualages { get; set; }

        private ArrayList _alAssociations;
        private ArrayList _alCloseCategoriesGroup;
        private ArrayList _alFiltrResults;
        private ArrayList _alTypeTimers;
     
        private static Codebooks instance = null;

        private Codebooks()
        {
            this.dAssociations = this.GetAssociations(@"CB_Associations.csv");
            this.dCategories = this.GetCategories(@"CB_Categories.csv");
            this.lLegualages = this.GetLeagues(@"CB_Leagues.csv");
            this.dTypeTimers = this.GetTypeTimers(@"CB_TypeTimers.csv");
        }

        public static Codebooks GetInstance()
        {
            if (instance == null)
            {
                instance = new Codebooks();
            }
            return instance;
        }

        private IDictionary<int, Association> GetAssociations(string fileName)
        {
            IDictionary<int, Association> dAssociations = new Dictionary<int, Association>();
            var reader = new StreamReader(fileName);
            while (!reader.EndOfStream)
            {
                String line = reader.ReadLine();
                String[] values = line.Split(';');

                Association association = new Association();
                association.name = values[1];
                association.district = values[2];
                dAssociations.Add(Convert.ToInt32(values[0]), association);
            }
            return dAssociations;
        }

        private IDictionary<int, Category> GetCategories(string fileName)
        {
            IDictionary<int, Category> dCategories = new Dictionary<int, Category>();
            var reader = new StreamReader(fileName);

            while (!reader.EndOfStream)
            {
                String line = reader.ReadLine();
                String[] values = line.Split(';');

                if (values[0] == "Y")
                {
                    Category categorie = new Category();
                    categorie.name = values[2];
                    categorie.groupId = Convert.ToInt32(values[3]);
                    categorie.groupName = values[4];
                    categorie.disciplineId = Convert.ToInt32(values[5]);
                    dCategories.Add(Convert.ToInt32(values[1]), categorie);
                }
            }
            return dCategories;
        }

        private IList<string> GetLeagues(string fileName)
        {
            IList<string> lLeagues = new List<string>();
            var reader = new StreamReader(fileName);

            while (!reader.EndOfStream)
            {
                String line = reader.ReadLine();
                String[] values = line.Split(';');

                if (values[0] == "Y")
                {
                    lLeagues.Add(values[1]);
                }
            }

            return lLeagues;
        }

        private IDictionary<int, TimerType> GetTypeTimers(string fileName)
        {
            IDictionary<int, TimerType> dTypeTimers = new Dictionary<int, TimerType>();
            var reader = new StreamReader(fileName);

            while (!reader.EndOfStream)
            {
                String line = reader.ReadLine();
                String[] values = line.Split(';');
                TimerType timerType = new TimerType();
                if (values[0] == "Y")
                {
                    timerType.name = values[2];
                    timerType.baudRate = Convert.ToInt32(values[3]);
                    dTypeTimers.Add(Convert.ToInt32(values[1]), timerType);
                }
            }
            return dTypeTimers;
        }

        public ArrayList GetAssociations()
        {
            if (this._alAssociations == null)
            {
                this._alAssociations = new ArrayList();
                foreach (var association in this.dAssociations.OrderBy(item => item.Value.name))
                {
                    this._alAssociations.Add(new KeyValuePair<int,string>(association.Key, association.Value.name + " (" + association.Value.district + ")"));
                }
            }

            return this._alAssociations;
        }

        public ArrayList GetCloseCategoriesGroup()
        {
            if (this._alCloseCategoriesGroup == null)
            {
                this._alCloseCategoriesGroup = new ArrayList();
                foreach (var item in this.dCategories)
                {
                    bool isExist = false;
                    foreach (KeyValuePair<int,string> existItem in this._alCloseCategoriesGroup.ToArray())
                    {
                        if (existItem.Key == item.Value.groupId)
                        {
                            isExist = true;
                            break;
                        }
                    }

                    if (!isExist)
                    {
                        this._alCloseCategoriesGroup.Add(new KeyValuePair<int,string>(item.Value.groupId, item.Value.groupName));
                    }
                }
            }
            return this._alCloseCategoriesGroup;
        }

        public ArrayList GetFiltrResults()
        {
            if (this._alFiltrResults == null)
            {
                this._alFiltrResults = new ArrayList();
                this._alFiltrResults.Add(new KeyValuePair<int,string>(0, "Výsledky všech katogorií"));
                foreach (var item in this.dCategories)
                {
                    this._alFiltrResults.Add(new KeyValuePair<int,string>(item.Key, item.Value.name));
                }
            }
            return this._alFiltrResults;
        }

        public ArrayList GetTypeTimers()
        {
            if (this._alTypeTimers == null)
            {
                this._alTypeTimers = new ArrayList();
                foreach (var item in this.dTypeTimers)
                {
                    this._alTypeTimers.Add(new KeyValuePair<int,string>(item.Key, item.Value.name));
                }
            }
            return this._alTypeTimers;
        }
        
        public IList<int> GetCloseCategories(int groupCategoryKey)
        {
            IList<int> lCloseCategories = new List<int>();

            foreach (var item in this.dCategories)
            {
                if (item.Value.groupId == groupCategoryKey)
                    lCloseCategories.Add(item.Key);
            }
            return lCloseCategories;
        }

        public int GetDisciplineIdByGroupCategoriesKey(int groupCategorykey)
        {
            foreach (var item in this.dCategories)
            {
                if (item.Value.groupId == groupCategorykey)
                    return item.Value.disciplineId;
            }
            return 0;
        }
    }
}