﻿using HtmlAgilityPack;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;

namespace Interconnection
{
    public class FiresportConnector
    {
        private static FiresportConnector instance = null;
        private HttpClient _client;
        private string _endPoint;
        private CookieContainer _cookieContainer;
        private HttpClientHandler _handler;
        private Uri _baseAddress;

        private FiresportConnector(string domain)
        {
            SetVariables(domain);
        }

        public static FiresportConnector GetInstance(string domain)
        {
            if (instance == null)
                instance = new FiresportConnector(domain);
            else
                instance.SetVariables(domain);

            return instance;
        }

        public static string CreateEndPoint(string domain)
        {
            return "https://" + domain + ".firesport.eu";
        }

        private void SetVariables(string domain)
        {
            _endPoint = CreateEndPoint(domain);
            _baseAddress = new Uri(_endPoint);
            _cookieContainer = new CookieContainer();
            _handler = new HttpClientHandler() { CookieContainer = _cookieContainer };
            _client = new HttpClient(_handler, false) { BaseAddress = _baseAddress };
        }

        public void Login(String email, String password)
        {
            var content = new FormUrlEncodedContent(new[]
            {
                    new KeyValuePair<string, string>("akce", "logine"),
                    new KeyValuePair<string, string>("email", email),
                    new KeyValuePair<string, string>("heslo",password)
                });

            var result = _client.PostAsync("/web_personal.php", content).Result;
            result.EnsureSuccessStatusCode();
            IEnumerable<Cookie> responseCookies = _cookieContainer.GetCookies(new Uri(_endPoint + "/web_personal.php")).Cast<Cookie>();
            foreach (Cookie cookie in responseCookies)
            {
                if (cookie.Name == "coopers")
                {
                    if (cookie.Value.Substring(cookie.Value.Length - 6) == "%3D%3D")
                        throw new InvalidCastException("Login failed. Invalid email or password");

                }
            }

        }

        [Obsolete("Method is deprecated, please use addResult.")]
        public void ImoportResults(int id, String data)
        {
            var content = new FormUrlEncodedContent(new[]
                {
                    new KeyValuePair<string, string>("soubor", data),
                    new KeyValuePair<string, string>("ids", id.ToString()),
                    new KeyValuePair<string, string>("aeids", id.ToString()),
                    new KeyValuePair<string, string>("akce", "feditvysledky")
                });
            var result = _client.PostAsync("web_vysledky.php#vysinport", content).Result;
        }

        [Obsolete("Method is deprecated, please use addResult.")]
        public void SaveResults(int idCompetion, int idCategory, int idTypCompetion, String data)
        {
            var content = new FormUrlEncodedContent(new[]
            {
                    new KeyValuePair<string, string>("teidk", idCategory.ToString()),
                    new KeyValuePair<string, string>("typ", idTypCompetion.ToString()),
                    new KeyValuePair<string, string>("kolo", "1"),
                    new KeyValuePair<string, string>("akce", "fvyssoubor"),
                    new KeyValuePair<string, string>("ids", idCompetion.ToString()),
                    new KeyValuePair<string, string>("aeids", idCompetion.ToString() ),
                    new KeyValuePair<string, string>("soubor", data)
                });
            var result = _client.PostAsync("web_vysledky.php#kontrolainport", content).Result;
        }

        [Obsolete("Method is deprecated, please use addResult.")]
        public void LoadIntoResults(int idCompetion)
        {
            var content = new FormUrlEncodedContent(new[]
            {
                new KeyValuePair<string, string>("akce", "fvysaktual"),
                new KeyValuePair<string, string>("ids", idCompetion.ToString() ),
                new KeyValuePair<string, string>("aeids", idCompetion.ToString())
            });
            var result = _client.PostAsync("web_vysledky.php#vysledky", content).Result;
        }

        public void AddResult(int competionId, int categoryId, int disciplineId, int firstTime, int secondTime, int order, int associationId, int point, string teamName)
        {
            int endTime = firstTime;
            if (firstTime < secondTime)
                endTime = secondTime;

            var content = new FormUrlEncodedContent(new[]
               {
                   new KeyValuePair<string, string>("idres", "1"),
                   new KeyValuePair<string, string>("kde", competionId.ToString() ),
                   new KeyValuePair<string, string>("kategor", categoryId.ToString()),
                   new KeyValuePair<string, string>("Typ", disciplineId.ToString()),
                   new KeyValuePair<string, string>("kdou", teamName),
                   new KeyValuePair<string, string>("kolo", "1"),
                   new KeyValuePair<string, string>("osoba", "0"),
                   new KeyValuePair<string, string>("cas1", firstTime.ToString()),
                   new KeyValuePair<string, string>("cas2", secondTime.ToString()),
                   new KeyValuePair<string, string>("kporadi", order.ToString()),
                   new KeyValuePair<string, string>("sosbor", associationId.ToString()),
                   new KeyValuePair<string, string>("cas", endTime.ToString()),
                   new KeyValuePair<string, string>("kbody", point.ToString()),
                   new KeyValuePair<string, string>("akce", "addv")
                });
            var result = _client.PostAsync("web_poradatel.php", content).Result;
        }

        public void EditResult(int runId, int firstTime, int secondTime, int order, int associationId, int point, int pointOrder, string teamName)
        {
            int endTime = firstTime;
            if (firstTime < secondTime)
                endTime = secondTime;

            var content = new FormUrlEncodedContent(new[]
               {
                    new KeyValuePair<string, string>("ikolo", "1"),
                    new KeyValuePair<string, string>("kporadi", order.ToString()),
                    new KeyValuePair<string, string>("kdruzstvo", order.ToString()),
                    new KeyValuePair<string, string>("typid", associationId.ToString()),
                    new KeyValuePair<string, string>("cas1", firstTime.ToString()),
                    new KeyValuePair<string, string>("cas2", secondTime.ToString()),
                    new KeyValuePair<string, string>("cas", endTime.ToString()),
                    new KeyValuePair<string, string>("kbody", point.ToString()),
                    new KeyValuePair<string, string>("pbody", pointOrder.ToString()),
                    new KeyValuePair<string, string>("osoba", "0"),
                    new KeyValuePair<string, string>("kidv", runId.ToString()),
                    new KeyValuePair<string, string>("akce", "ediv")
                });
            var result = _client.PostAsync("web_vysledky.php", content).Result;
        }

        public void SetResultToLig(int competionId, int runId, bool inLig)
        {
            String action = "neliga";
            if (inLig)
                action = "anoliga";

            var content = new FormUrlEncodedContent(new[]
               {
                    new KeyValuePair<string, string>("akce", action),
                    new KeyValuePair<string, string>("ids", competionId.ToString()),
                    new KeyValuePair<string, string>("idv", runId.ToString())

                });
            var result = _client.PostAsync("web_vysledky.php", content).Result;
        }

        public void DeleteResult(int competionId, int runId)
        {
            var content = new FormUrlEncodedContent(new[]
               {
                    new KeyValuePair<string, string>("akce", "delv"),
                    new KeyValuePair<string, string>("ids", competionId.ToString()),
                    new KeyValuePair<string, string>("idv", runId.ToString())

                });
            var result = _client.PostAsync("web_vysledky.php", content).Result;
        }

        public IList<Competion> GetCompetions(int year)
        {
            HttpResponseMessage response = _client.GetAsync("web_souteze.php?akce=sel&rok=" + year.ToString()).Result;
            response.EnsureSuccessStatusCode();
            string responseBody = response.Content.ReadAsStringAsync().Result.ToString();

            HtmlDocument doc = new HtmlDocument();
            doc.LoadHtml(responseBody);
            HtmlNodeCollection col = doc.DocumentNode.SelectNodes("//html/body//div[1]//div[3]//div[1]//div[2]//div[1]//div[1]//ul//li");
            
            IList<Competion> lCompetion = new List<Competion>();
            String title;
            String place;
            int competionId;

            foreach (HtmlNode node in col)
            {
                title = node.FirstChild.Attributes.Where(a => a.Name == "title").ToList()[0].Value;
                place = node.FirstChild.FirstChild.InnerHtml;
                competionId = Convert.ToInt32(node.FirstChild.Attributes.Where(a => a.Name == "href").ToList()[0].Value.Split(new char[] { '-' })[1]);
                lCompetion.Add(new Competion(competionId, title, place));
            }

            return lCompetion;
        }
    }
}
