﻿using System;
using System.Collections.Generic;
using System.IO.Ports;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Interconnection
{
    public class SerialComunicator
    {
        private SerialPort _port;
        private static SerialComunicator instance;

        private SerialComunicator(string portName, int baudRate)
        {
            this._port = new SerialPort(portName, baudRate);
            this._port.Encoding = Encoding.GetEncoding("iso-8859-1");
            this._port.ReadTimeout = 100;
            this._port.WriteTimeout = 100;
            this._port.NewLine = "\r";
            this._port.Open();
        }

        public static SerialComunicator GetInstance(string portName, int baudRate)
        {
            if (instance == null)
            {
                instance = new SerialComunicator(portName, baudRate);
            }
            else
            {
                ChangeSettigns(portName, baudRate);
            }
            return instance;
        }

        private static void ChangeSettigns(string portName, int baudRate)
        {
            instance._port.Close();
            instance._port.PortName = portName;
            instance._port.BaudRate = baudRate;
            instance._port.Open();
        }

        public string SendAndRead(string dataToSend)
        {
            if (this._port == null || !this._port.IsOpen)
                return "";
            this._port.ReadTimeout = 100;
            this._port.Write("#" + dataToSend + "\n");
            try
            {
                return this._port.ReadLine();
            }
            catch
            {
                return "";
            }
        }

        public void Reset()
        {
            this.Write("#RST\n");
        }

        public void Change()
        {
            this.SendAndRead("APP:cw:change");
        }

        public void Write(string dataToSend)
        {
            this._port.Write(dataToSend);
        }

        public string Read()
        {
            if (this._port.IsOpen)
            {
                this._port.Write("#APP:cw:data?\n");

                try
                {
                    return this._port.ReadLine();
                }
                catch
                {
                    return "";
                }
            }
            return "";
        }
    }
}
