﻿using Interconnection;
using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Windows.Forms;

namespace Timer
{
    class CustomizationUIComponet
    {
        private const int TVIF_STATE = 0x8;
        private const int TVIS_STATEIMAGEMASK = 0xF000;
        private const int TV_FIRST = 0x1100;
        private const int TVM_SETITEM = TV_FIRST + 63;

        [StructLayout(LayoutKind.Sequential, Pack = 8, CharSet = CharSet.Auto)]
        private struct TVITEM
        {
            public int mask;
            public IntPtr hItem;
            public int state;
            public int stateMask;
            [MarshalAs(UnmanagedType.LPTStr)]
            public string lpszText;
            public int cchTextMax;
            public int iImage;
            public int iSelectedImage;
            public int cChildren;
            public IntPtr lParam;
        }

        [DllImport("user32.dll", CharSet = CharSet.Auto)]
        private static extern IntPtr SendMessage(IntPtr hWnd, int Msg, IntPtr wParam, ref TVITEM lParam);

        private void HideCheckBox(TreeView tvw, TreeNode node)
        {
            TVITEM tvi = new TVITEM();
            tvi.hItem = node.Handle;
            tvi.mask = TVIF_STATE;
            tvi.stateMask = TVIS_STATEIMAGEMASK;
            tvi.state = 0;
            SendMessage(tvw.Handle, TVM_SETITEM, IntPtr.Zero, ref tvi);
        }

        public void SetTreeViewCategories( TreeView treeViewCategories, IDictionary<int, Interconnection.Category> dCategories)
        {
            TreeNode treeNodeParent = new TreeNode("");
            
            treeViewCategories.Nodes.Add(treeNodeParent);
            this.HideCheckBox(treeViewCategories, treeNodeParent);

            foreach (var item in dCategories)
            {
                TreeNode[] treeNodeChilds;
                TreeNode treeNodeA = new TreeNode("A");
                treeNodeA.Name = item.Value.name + " A";
                treeNodeA.Tag = item.Key;

                TreeNode treeNodeB = new TreeNode("B");
                treeNodeB.Name = item.Value.name + " B";
                treeNodeB.Tag = item.Key;

                if (item.Key == 1 || item.Key == 2 || item.Key == 16)
                {
                    treeNodeChilds = new TreeNode[] { treeNodeA, treeNodeB };

                }
                else
                {
                    TreeNode treeNodeC = new TreeNode("C");
                    treeNodeC.Name = item.Value.name + " C";
                    treeNodeC.Tag = item.Key;

                    treeNodeChilds = new TreeNode[] { treeNodeA, treeNodeB, treeNodeC };
                }

                treeNodeParent = new TreeNode(item.Value.name, treeNodeChilds);
                treeNodeParent.Name = item.Key.ToString();

                treeViewCategories.Nodes.Add(treeNodeParent);
                this.HideCheckBox(treeViewCategories, treeNodeParent);

            }
            treeViewCategories.ExpandAll();
        }

        private bool ContainSettingsGroup(IDictionary<string, IDictionary<int, string>> dCategorieGroup, int key)
        {

            foreach (var item in dCategorieGroup)
            {
                if (item.Value.ContainsKey(key))
                    return true;
            }
            return false;
        }

        public void SetTreeViewSettingsGroupsCategories(ref TreeView treeViewSetingsGroupsCategories, IDictionary<string, IDictionary<int, string>> dCategorieGroup)
        {

            treeViewSetingsGroupsCategories.Nodes.Clear();
            foreach (var itemGroup in dCategorieGroup)
            {
                TreeNode[] treeNodeChilds = new TreeNode[itemGroup.Value.Count];
                int iterator = 0;
                foreach (var itemCategory in itemGroup.Value)
                {
                    TreeNode treeNode = new TreeNode(itemCategory.Value);
                    treeNode.Name = itemCategory.Value;
                    treeNode.Tag = itemCategory.Key;
                    treeNodeChilds[iterator++] = treeNode;
                }

                TreeNode treeNodeParent = new TreeNode(itemGroup.Key, treeNodeChilds);
                treeNodeParent.Name = itemGroup.Key;

                treeViewSetingsGroupsCategories.Nodes.Add(treeNodeParent);
                this.HideCheckBox(treeViewSetingsGroupsCategories, treeNodeParent);
            }
            treeViewSetingsGroupsCategories.ExpandAll();
        }

        public void UpdateTreeViewCategories( TreeView treeViewCategories, int categoryValue)
        {
            foreach (TreeNode treeNodeParent in treeViewCategories.Nodes)
            {
                if (treeNodeParent.Name == categoryValue.ToString())
                {
                    foreach (TreeNode treeNodeChild in treeNodeParent.Nodes)
                    {
                        HideCheckBox(treeViewCategories, treeNodeChild);
                    }
                }
            }
        }

        private void AddTreeViewPrezentation( TreeView treeViewPrezentation, Association association)
        {
            TreeNode[] treeNodeChilds = new TreeNode[association.dTeam.Count];
            IList<TreeNode> ltreeNodeDisableCheck = new List<TreeNode>();
            int i = 0;
            foreach (KeyValuePair<string, Team> team in association.dTeam)
            {
                treeNodeChilds[i] = new TreeNode(team.Key);
                if (team.Value.prezentationClose)
                    ltreeNodeDisableCheck.Add(treeNodeChilds[i]);
                i++;
            }
            TreeNode treeNodeParent = new TreeNode(association.name, treeNodeChilds);
            treeViewPrezentation.Nodes.Add(treeNodeParent);

            foreach (TreeNode treeNodeDisableCheck in ltreeNodeDisableCheck)
            {
                HideCheckBox(treeViewPrezentation, treeNodeDisableCheck);
            }
        }

        public void ReloadTreeViewPrezentation( TreeView treeViewPrezentation, IList<Association> lAssociations)
        {
            treeViewPrezentation.Nodes.Clear();
            foreach (Association association in lAssociations)
            {
                AddTreeViewPrezentation( treeViewPrezentation, association);
            }
        }
    }
}
