﻿using Interconnection;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Timer
{
    public class ExportResults
    {

        public static void ToFiresport(IList<Team> lResults, Settings settings)
        { 
            if (lResults.Count == 0)
            {
                MessageBox.Show("Nejsou žádné výsledky k nahrání na webové stránky firespotr.");
            }
            else
            {
                FiresportConnector firesportConnector = FiresportConnector.GetInstance(settings.currentLeguage);
                firesportConnector.Login(settings.firesportLogin, settings.firesportPassword);

                foreach (var team in lResults)
                {
                    Time time = team.lTimes.OrderBy(item => item.final.time).FirstOrDefault();
                    if (time.final.isValid)
                        firesportConnector.AddResult(settings.firesportCompetitionId, team.firesportCategoryId, team.disciplineId, Time.RemoveMillisecond(time.left.time), Time.RemoveMillisecond(time.right.time), team.order, team.firesportId, team.points, team.name);
                    else
                    {
                        time = new Time();
                        firesportConnector.AddResult(settings.firesportCompetitionId, team.firesportCategoryId, team.disciplineId, Time.RemoveMillisecond(time.left.time), Time.RemoveMillisecond(time.right.time), team.order, team.firesportId, team.points, team.name);
                    }
                }
                MessageBox.Show("Výsledky byly nahrány na portál firesport.");

            }
        }

        public static void ToCSV(IList<Team> lResults)
        {
            if (lResults.Count == 0)
            {
                MessageBox.Show("Nejsou žádné výsledky k exortu do CSV.");
            }
            else
            {
                string filePath = Environment.GetFolderPath(Environment.SpecialFolder.Desktop);

                filePath = filePath + @"\Výsledky-" + DateTime.Now.ToString("yyyy-dd-M-HH-mm-ss") + ".csv";
                TextWriter sw = new StreamWriter(filePath, false, Encoding.UTF8);
                Time finalTime = null;
                string write = "";

                for (int i = 1; i <= lResults[0].lTimes.Count; i++)
                {
                    write = write + i.ToString() + ". kolo platný pokus;" +
                         i.ToString() + ". kolo výsledný čas;" +
                          i.ToString() + ". kolo levý terč;" +
                           i.ToString() + ". kolo pravý terč;";
                }
                sw.WriteLine("FiresportId;Popis týmu; Kategorie Id;Typ závodu Id;Pořadí;Platný pokus;Výsledný čas;Levý terč;Pravý terč;" + write);
                foreach (var team in lResults)
                {
                    finalTime = team.lTimes.Where(item => item.final.isValid == true).OrderBy(item => item.final.time).FirstOrDefault();
                    if (finalTime == null)
                    {
                        finalTime = team.lTimes.
                            OrderBy(item => item.final.time).
                            ThenBy(item => item.left.time).
                            ThenBy(item => item.right.time).
                            FirstOrDefault();

                        write = "false;"
                             + "NP;"
                             + Time.TransformTime(finalTime.left.time) + ";"
                             + Time.TransformTime(finalTime.right.time) + ";";
                    }
                    else
                    {
                        write = finalTime.final.isValid.ToString() + ";"
                            + Time.TransformTime(finalTime.final.time) + ";"
                            + Time.TransformTime(finalTime.left.time) + ";"
                            + Time.TransformTime(finalTime.right.time) + ";";

                    }
                    foreach (var time in team.lTimes)
                    {
                        if (time.final.isValid == false)
                        {
                            write = write + time.final.isValid.ToString() + ";"
                                + "NP;";
                            if (time.left.time == 120000)
                                write = write + "NP;";
                            else
                                write = write + Time.TransformTime(time.left.time) + ";";

                            if (time.right.time == 120000)
                                write = write + "NP;";
                            else
                                write = write + Time.TransformTime(time.right.time) + ";";

                        }
                        else
                        {
                            write = write + time.final.isValid.ToString() + ";"
                                + Time.TransformTime(time.final.time) + ";"
                                + Time.TransformTime(time.left.time) + ";"
                                + Time.TransformTime(time.right.time) + ";";
                        }
                    }

                    sw.WriteLine(team.firesportId + ";"
                        + team.description.Split(new string[] { " - " }, StringSplitOptions.None)[1] + ";"
                        + team.firesportCategoryId + ";"
                        + team.disciplineId + ";"
                        + team.order + ";"
                        + write);
                }

                sw.Close();
                MessageBox.Show("Výsledky byly exportováný do " + filePath);
            }
        }
    }
}
