﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Timer
{
    public partial class FormEditResult : Form
    {
        private Team _team;

        public FormEditResult(Team team)
        {
            InitializeComponent();

            this._team = team;
            this.labelDescription.Text = this._team.TrimDescription();
            if (team.lTimes.Count < 4)
            {
                this.DisableRoundElements(this.checkBoxFourthRoundValid, this.numericUpDownFourthRoundLeft,
                     this.numericUpDownFourthRoundRight);
                if (team.lTimes.Count < 3)
                    this.DisableRoundElements(this.checkBoxThirdRoundValid, this.numericUpDownThirdRoundLeft,
                        this.numericUpDownThirdRoundRight);
                if (team.lTimes.Count < 2)
                    this.DisableRoundElements(this.checkBoxSecondRoundValid, this.numericUpDownSecondRoundLeft,
                        this.numericUpDownSecondRoundRight);
            }

            this.SetRoundFields(this._team.lTimes[0], this.checkBoxFirstRoundValid, this.numericUpDownFirstRoundLeft,
                this.numericUpDownFirstRoundRight, this.labelFirstRoundLeftResult, this.labelFirstRoundRightResult);

            if (team.lTimes.Count > 1)
            {
                this.SetRoundFields(this._team.lTimes[1], this.checkBoxSecondRoundValid, this.numericUpDownSecondRoundLeft,
                    this.numericUpDownSecondRoundRight, this.labelSecondRoundLeftResult, this.labelSecondRoundRightResult);

                if (team.lTimes.Count > 2)
                    this.SetRoundFields(this._team.lTimes[2], this.checkBoxThirdRoundValid, this.numericUpDownThirdRoundLeft,
                   this.numericUpDownThirdRoundRight, this.labelThirdRoundLeftResult, this.labelThirdRoundRightResult);

                if (team.lTimes.Count > 3)
                    this.SetRoundFields(this._team.lTimes[3], this.checkBoxFourthRoundValid, this.numericUpDownFourthRoundLeft,
                                 this.numericUpDownFourthRoundRight, this.labelFourthRoundLeftResult, this.labelFourthRoundRightResult);
            }
        }

        private void DisableRoundElements(CheckBox checkBoxRoundValid, NumericUpDown numericUpDownRoundLeft,
            NumericUpDown numericUpDownRoundRight)
        {
            checkBoxRoundValid.Enabled = false;
            numericUpDownRoundLeft.Enabled = false;
            numericUpDownRoundRight.Enabled = false;
        }

        private void SetRoundFields(Time time, CheckBox checkBoxRoundValid, NumericUpDown numericUpDownRoundLeft,
            NumericUpDown numericUpDownRoundRight, Label labelRoundLeftResult, Label labelRoundRightResult)
        {
            SetTimeTarget(Time.RemoveMillisecond(time.left.time), numericUpDownRoundLeft, labelRoundLeftResult, checkBoxRoundValid);
            SetTimeTarget(Time.RemoveMillisecond(time.right.time), numericUpDownRoundRight, labelRoundRightResult, checkBoxRoundValid);
        }

        private Time GetRoundFields(Time time, CheckBox checkBoxRoundValid, NumericUpDown numericUpDownRoundLeft,
           NumericUpDown numericUpDownRoundRight)
        {
            time.left.time = (int)numericUpDownRoundLeft.Value * 10;
            time.right.time = (int)numericUpDownRoundRight.Value * 10;

            if (time.left.time > time.right.time)
                time.final.time = time.left.time;
            else
                time.final.time = time.right.time;

            time.final.isValid = checkBoxRoundValid.Checked;
            checkBoxRoundValid.Checked = time.final.isValid;

            return time;
        }

        private void SetTimeTarget(int value, NumericUpDown numericUpDownRound, Label labelRound, CheckBox checkBox)
        {
            numericUpDownRound.Value = value;
            value = value * 10;
            labelRound.Text = Time.TransformTime((value));
            checkBox.Checked = TimeDetail.IsValidTime(value);
        }

        public Team GetFormData()
        {

            this.GetRoundFields(this._team.lTimes[0], this.checkBoxFirstRoundValid, this.numericUpDownFirstRoundLeft,
                   this.numericUpDownFirstRoundRight);
            if (this._team.lTimes.Count > 1)
            {
                this.GetRoundFields(this._team.lTimes[1], this.checkBoxSecondRoundValid, this.numericUpDownSecondRoundLeft,
                    this.numericUpDownSecondRoundRight);

                if (this._team.lTimes.Count > 2)
                    this.GetRoundFields(this._team.lTimes[2], this.checkBoxThirdRoundValid, this.numericUpDownThirdRoundLeft,
                        this.numericUpDownThirdRoundRight);

                if (this._team.lTimes.Count > 3)
                    this.GetRoundFields(this._team.lTimes[3], this.checkBoxFourthRoundValid, this.numericUpDownFourthRoundLeft,
                        this.numericUpDownFourthRoundRight);
            }

            return this._team;
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void buttonOK_Click(object sender, EventArgs e)
        {
            this.DialogResult = System.Windows.Forms.DialogResult.OK;
            this.Close();
        }

        private void numericUpDownFirstRoundLeft_ValueChanged(object sender, EventArgs e)
        {
            this.SetTimeTarget((int)(this.numericUpDownFirstRoundLeft.Value), this.numericUpDownFirstRoundLeft, this.labelFirstRoundLeftResult, this.checkBoxFirstRoundValid);
        }

        private void numericUpDownFirstRoundRight_Click(object sender, EventArgs e)
        {
            SetTimeTarget((int)this.numericUpDownFirstRoundRight.Value, this.numericUpDownFirstRoundRight, this.labelFirstRoundRightResult, this.checkBoxFirstRoundValid);
        }

        private void numericUpDownSecondRoundLeft_Click(object sender, EventArgs e)
        {
            SetTimeTarget((int)this.numericUpDownSecondRoundLeft.Value, this.numericUpDownSecondRoundLeft, this.labelSecondRoundLeftResult, this.checkBoxSecondRoundValid);
        }

        private void numericUpDownSecondRoundRight_Click(object sender, EventArgs e)
        {
            SetTimeTarget((int)this.numericUpDownSecondRoundRight.Value, this.numericUpDownSecondRoundRight, this.labelSecondRoundRightResult, this.checkBoxSecondRoundValid);
        }

        private void numericUpDownThirdRoundLeft_Click(object sender, EventArgs e)
        {
            SetTimeTarget((int)this.numericUpDownThirdRoundLeft.Value, this.numericUpDownThirdRoundLeft, this.labelThirdRoundLeftResult, this.checkBoxThirdRoundValid);
        }

        private void numericUpDownThirdRoundRight_Click(object sender, EventArgs e)
        {
            SetTimeTarget((int)this.numericUpDownThirdRoundRight.Value, this.numericUpDownThirdRoundRight, this.labelThirdRoundRightResult, this.checkBoxThirdRoundValid);
        }

        private void numericUpDownFourthRoundLeft_Click(object sender, EventArgs e)
        {
            SetTimeTarget((int)this.numericUpDownFourthRoundLeft.Value, this.numericUpDownFourthRoundLeft, this.labelFourthRoundLeftResult, this.checkBoxFourthRoundValid);
        }

        private void numericUpDownFourthRoundRight_Click(object sender, EventArgs e)
        {
            SetTimeTarget((int)this.numericUpDownFourthRoundRight.Value, this.numericUpDownFourthRoundRight, this.labelFourthRoundRightResult, this.checkBoxFourthRoundValid);
        }
    }
}
